-- local _, features = pcall(require, "features")
require "features"
local pl = require "pl.pretty"
local lfs = require "lfs"

local _M = {}
local config = {
    [0] = {},
    [1] = {},
    ["disk"] = {},
}
local StreamsMap = {
	[0] = "main",
	[1] = "sub"
}

local function GetCoAFProxy( objPath, service, loc, timeout )
	local client = require "coaf.client"
	local proxy
	local cli
	cli = client.New( service, loc or "local", nil, timeout or 3 ) --3 seconds timeout.
	if not cli then
		print( "WEB", "Fail to create connection to service: " .. (loc or "local") .. "/" .. (service or "nil") )
		return nil
	end
	proxy = cli:CreateProxy(objPath, true) --Exclusive
	if not proxy then
		print( "WEB", "Fail to create object proxy for '" .. (objPath or "unknown") .. "'" )
		return nil
	end
	return proxy
end

function _M.Init()
    print("####### in function Init !!!!!!!!!")
end

function _M.getDiskInfo()
	local proxy = GetCoAFProxy( "/", "record", nil, 600 )
    if proxy then
        local done,main,sub,disk = proxy:getDiskInfo()
        if done then
            config[0] = main
            config[1] = sub
            config["disk"] = disk
        end
        proxy:Destroy()
    end
end

function _M.ftpUploadCheck()
    print("######### ftp Upload Check !!!!!!!!!")
    _M.getDiskInfo()
    local k,v
    local main = false
    local sub = false
	for k,v in pairs(StreamsMap) do
		if config[k].auto_upload ~= 0 then	--Auto start recording, so try start it
			local time = os.time()
			local n_hour,n_minute
			local upload_hour,upload_min = config[k].upload_time_hour, config[k].upload_time_minute
			n_hour = os.date("%H",time)
			n_minute = os.date("%M",time)
			print("FTP arguments:",upload_hour,upload_min,n_hour,n_minute)
			if n_hour+0 == upload_hour+0 and n_minute+0 == upload_min+0 then
                if k == 0 then
                    main = true
                else
                    sub = true
                end
			end
		end
	end
    if main or sub then
        local i,_v
        for i,_v in pairs(config["disk"]) do
            local folder = _v["MOUNT"] .. "/RECORD"
            _M.recurLoadfile(folder,main,sub)
        end
    end
end

function _M.recurLoadfile(folder, main, sub)
    print("FTP folder is " .. folder)
    local f,file
    _M.uploadStatus = false
    for f in lfs.dir(folder) do
        if lfs.attributes(folder.."/"..f).mode == "directory" then
            -- local fileCounts = 0
            -- local uploadCounts = 0
            for file in lfs.dir(folder.."/"..f) do
                if file ~= "." and file ~= ".." and string.match(file, "()%.") ~= 1 then
                    local attr = lfs.attributes(folder .."/"..f.."/"..file)
                    local canUplod = io.open(folder.."/"..f.."/."..file..".UPLOAD","r")
                    if attr and attr.mode == "file" and canUplod then
                        -- fileCounts = fileCounts + 1
                        canUplod:close()
                        local cmd = nil
                        if string.match(f,"^(%u)") == "M" and main then
                            cmd = "/usr/bin/ftpput -u " .. config[0].user .. " -p " .. config[0].password .. " " .. 
                                config[0].host .. " " .. config[0].path .. "/" .. file .. " " .. folder .. "/" .. f .. "/" .. file --.. " &"
                        elseif string.match(f,"^(%u)") == "S" and sub then
                            cmd = "/usr/bin/ftpput -u " .. config[1].user .. " -p " .. config[1].password .. " " .. 
                                config[1].host .. " " .. config[1].path .. "/" .. file .. " " .. folder .. "/" .. f .. "/" .. file --.. " &"
                        end
                        if cmd then
                            print("FTP cmd is:" .. cmd)
                            local r,err = os.execute(cmd)
                            if not r then
                                print("FTP Upload " .. file .. " Fail: " .. err)
                                _M.uploadStatus = true 
                                return false
                            else
                                -- uploadCounts = uploadCounts + 1
                                os.execute("/bin/rm " .. folder.."/"..f.."/."..file..".UPLOAD")
                                print( "FTP Upload " .. file .. " Success!")
                            end
                        end
                    end
                end
            end
            -- if fileCounts ~= 0 and fileCounts == uploadCounts then
            --     os.execute("/bin/rm " .. folder.."/"..f.."/.UPLOAD")
            -- end
        end
    end  
    _M.uploadStatus = true 
    return true
end

function _M.manualUpload(args)
    _M.getDiskInfo()
    local main = false
    local sub = false

    local i,v 
    for i,v in pairs(args) do
        local res = string.match(i, "%[(%w+)%]")
        if res then
            main = tonumber(res) == 0
            sub = tonumber(res) == 1
            break
        end
    end

    if main or sub then
        local _i,_v
        for _i,_v in pairs(config["disk"]) do
            local folder = _v["MOUNT"] .. "/RECORD"
            if not _M.recurLoadfile(folder,main,sub) then
                return false
            end
        end
    end
    return true
end

function _M.getUploadStatus()
    return _M.uploadStatus or false
end

return _M