#!/usr/bin/env lua

local coaf=require "coaf.core"
local _dbg = require "coaf.debug"

coaf.ChDir(coaf.GetAppDir(arg))

local socket = require "cosocket.core"

local client = require "coaf.client"
local adapter = require "coaf.cli_adapter.simple"
local service = require "coaf.service"

_G.print = function(...)
	_dbg.Info1( "FTP_UPLOAD", ... )
end

---- 日志模块
local withLog = false
for _x=1, #arg do
	if arg[_x] == "--log" then
		withLog = true
		break
	end
end

if withLog then
	-- ** initialize the log file **
	local roll = require "coaf.log.rollfile"
	os.execute( "/bin/mkdir -p /tmp/" )
	local logF = roll.Create("/tmp/ftp_service.log")
	if not logF or not logF:init() then
		_dbg.Error("FTP_UPLOAD", "Fail to open log file: /tmp/ftp_service.log, so the log messages should be output to console.")
		withLog = false
	else
		_dbg.AddLogger("ftp_service", logF, nil, nil)
	end
	-- ** log **
end

coaf.Init( "ftp_service" )

service.InitSocket()
client.InitSocket( socket, adapter )

local cosvc_ftp
local upload_timer
local mod_ftp = require "ftp_moudle"


local function uploadCheck()
    mod_ftp.ftpUploadCheck()
end

local function Main(args)
	cosvc_ftp = service.New( "ftp_service", "unix:/var/run/ftp_service.sock" )
	if cosvc_ftp then
		_dbg.Info( "FTP_UPLOAD", "CoService of 'ftp_upload' is created." )
		cosvc_ftp:CreateCoServerProxy()
	else
		_dbg.Error( "FTP_UPLOAD", "Fail created CoService!" )
	end

	mod_ftp.Init()
	cosvc_ftp:ExposeObject("/", mod_ftp )
	
	upload_timer = coaf.NewTimer( uploadCheck, 0.01, 20)--After 0.01 second, and per 50 seconds check.
	if upload_timer then
		upload_timer:start()
	end
end

coaf.Start(Main, arg)

_dbg.Print( "FTP_UPLOAD", "Service terminated." )